@@ -72,23 +72,33 @@ func finalExp(g *Gt, f *ff.Fp12) {
 	ff.HardExponentiation(&g.i, c)
 }
 
-// ProdPair calculates the product of pairings, i.e., \Prod_i pair(Pi,Qi)^ni.
+// ProdPair calculates the product of pairings:
+//
+//	e = \Prod_i pair(Pi, Qi)^ni
+//	  = \Prod_i pair(ni*Pi, Qi)
+//	  = \Prod_i pair(Pi, ni*Qi)
+//
+// For efficiency, it performs operations in G1.
 func ProdPair(P []*G1, Q []*G2, n []*Scalar) *Gt {
 	if len(P) != len(Q) || len(P) != len(n) {
 		panic("mismatch length of inputs")
 	}
 
-	ei := new(ff.Fp12)
+	scaled := make([]*G1, len(P))
+	for i := range P {
+		scaled[i] = new(G1)
+		scaled[i].ScalarMult(n[i], P[i])
+	}
+
+	affineP := affinize(scaled)
+
 	mi := new(ff.Fp12)
 	out := new(ff.Fp12)
 	out.SetOne()
 
-	affineP := affinize(P)
 	for i := range affineP {
 		miller(mi, &affineP[i], Q[i])
-		nb, _ := n[i].MarshalBinary()
-		ei.Exp(mi, nb)
-		out.Mul(out, ei)
+		out.Mul(out, mi)
 	}
 
 	e := &Gt{}
